/*
  Teensy MIDI controller for SysEx v1.2 by simon - [at] - brem.org
  Heavily inspired from Teensy 3 USB MIDI controller by Yann Seznec.
  Made for my custom MIDI-pedal with TeensyLC to control Arturia Beatstep 1, which plays drums on a RaspberryPi1 running samplerbox.org

  Changelog:
  -added an RGB LED for showing beat
  -bugfix in tap-to-tempo: Divider was wrong

  Licenced under CC BY-NC-SA 4.0:
  https://creativecommons.org/licenses/by-nc-sa/4.0/

  TODO:
  -tap-to-tempo-button to BPM conversion correct?
  -bug: tap-to-tempo disrupts tempo +/- function
  -maybe do something with Beatstep's LEDs

  NOTES:
  -Tested with TeensyLC-board and USB-Midi, but all Teensy-boards should work.
  -Remember to select MIDI as your USB Type in the Tools menu.
  -Don't forget to connect your varius USB-Midi-devices at the host (i.e. with aconnect).
  -If you use it with Beatstep 1, don't forget to select "external sync" on it.
  -Untested and not possible with Beatstep 1: If you want to use regular MIDI using din connectors, keep the Rx and Tx pins free. On Teensy LC and Teensy 3 (and up) these are pins 0 and 1.
  -Check this page for more info on how to wire this up: https://www.pjrc.com/teensy/td_libs_MIDI.html
*/

#include <MIDI.h>

IntervalTimer myTimer;

// total number of buttons:
int const numDigPins = 7;

// replace pin numbers if you use other pins for your buttons. Mind the order of this array:
int sysexpin[] =
{
  18, // preset up
  17, // preset down
  5,  // sequencer start
  6,  // sequencer stop
  14, // BPM up 10
  12, // BPM down 10
  9   // tap to tempo
};

#define LED_R 24
#define LED_G 23
#define LED_B 25

int debounce[numDigPins];

uint8_t presetnum;

const uint8_t start_seq[] = { 0xFA };
const uint8_t stop_seq[] = { 0xFC };

const uint8_t led_on[] = { 0x90, 0x24, 0x7F };
const uint8_t led_off[] = { 0x80, 0x24, 0x00 };

const int channel = 1; // not needed for sysex (?)

unsigned long bpm_us = 500000 / 16;
bool first_tap = 1;
unsigned long a_tap;
unsigned long b_tap;

MIDI_CREATE_DEFAULT_INSTANCE(); // won't compile without this!

int led_bpm_divider = 1;
int led_sequence = 1;

void setup()
{

  pinMode(LED_R, OUTPUT);
  pinMode(LED_G, OUTPUT);
  pinMode(LED_B, OUTPUT);
  
  for (int i = 0; i < numDigPins; i++)
  {
    pinMode(sysexpin[i], INPUT_PULLUP);
  }

  myTimer.begin(midi_clock, bpm_us);  // blinkLED to run every 0.15 seconds

  //  usbMIDI.sendSysEx(12, sel_mode, true);
  //  usbMIDI.sendSysEx(12, sel_c1, true);
}

void midi_clock()
{
  usbMIDI.sendRealTime(usbMIDI.Clock);

  if (led_bpm_divider == 1)
    {
    digitalWrite(LED_B, 1);
    led_bpm_divider++;

    if (led_sequence == 1)
      {
      digitalWrite(LED_G, 1);
      led_sequence++;
      }
    else
      led_sequence++;
    }
  else if (led_bpm_divider >= 2 && led_bpm_divider < 24)
    {
    digitalWrite(LED_G, 0);
    digitalWrite(LED_B, 0);
    digitalWrite(LED_R, 0);
    led_bpm_divider++;
    }
  else if (led_bpm_divider == 24)
    {
    led_bpm_divider = 1;
    }
  if (led_sequence == 5)
    led_sequence = 1;
}

void loop()
{

  // preset up
  if (digitalRead(sysexpin[0]) == 0 && debounce[0] == 0 && presetnum < 16)
  {
    presetnum++;
    const uint8_t preset[] = { 0xF0, 0x00, 0x20, 0x6B, 0x7F, 0x42, 0x05, presetnum, 0xF7 };
    usbMIDI.sendSysEx(9, preset, false);
    debounce[0] = 1;
    delay(5);
  }
  else if (digitalRead(sysexpin[0]) == 1 && debounce[0] == 1)
  {
    debounce[0] = 0;
    delay(5);
  }

  // preset down
  if (digitalRead(sysexpin[1]) == 0 && debounce[1] == 0 && presetnum > 1)
  {
    presetnum--;
    const uint8_t preset[] = { 0xF0, 0x00, 0x20, 0x6B, 0x7F, 0x42, 0x05, presetnum, 0xF7 };
    usbMIDI.sendSysEx(9, preset, false);
    debounce[1] = 1;
    delay(5);
  }
  else if (digitalRead(sysexpin[1]) == 1 && debounce[1] == 1)
  {
    debounce[1] = 0;
    delay(5);
  }

  // start seq
  if (digitalRead(sysexpin[2]) == 0 && debounce[2] == 0)
  {
    usbMIDI.sendSysEx(1, start_seq, false);
    led_bpm_divider = 1;
    led_sequence = 1;
    debounce[2] = 1;
    delay(5);
  }
  else if (digitalRead(sysexpin[2]) == 1 && debounce[2] == 1)
  {
    debounce[2] = 0;
    delay(5);
  }

  // stop seq
  if (digitalRead(sysexpin[3]) == 0 && debounce[3] == 0)
  {
    usbMIDI.sendSysEx(1, stop_seq, false);
    debounce[3] = 1;
    delay(5);
  }
  else if (digitalRead(sysexpin[3]) == 1 && debounce[3] == 1)
  {
    debounce[3] = 0;
    delay(5);
  }

  // bpm up
  if (digitalRead(sysexpin[4]) == 0 && debounce[4] == 0 && bpm_us > (200000 / 16))
  {
    bpm_us = bpm_us - (25000 / 16);
    myTimer.update(bpm_us);
    debounce[4] = 1;
    delay(5);
  }
  else if (digitalRead(sysexpin[4]) == 1 && debounce[4] == 1)
  {
    debounce[4] = 0;
    delay(5);
  }

  // bpm down
  if (digitalRead(sysexpin[5]) == 0 && debounce[5] == 0 && bpm_us < (1600000 / 16))
  {
    bpm_us = bpm_us + (25000 / 16);
    myTimer.update(bpm_us);
    debounce[5] = 1;
    delay(5);
  }
  else if (digitalRead(sysexpin[5]) == 1 && debounce[5] == 1)
  {
    debounce[5] = 0;
    delay(5);
  }

  // tap to tempo
  if (digitalRead(sysexpin[6]) == 0 && debounce[6] == 0)
  {
    if (first_tap == 1)
    {
      a_tap = micros();
      first_tap = 0;
    }
    else if (first_tap == 0)
    {
      b_tap = micros();
      bpm_us = (b_tap - a_tap) / 24;
      myTimer.update(bpm_us);
      first_tap = 1;
    }
    debounce[6] = 1;
    delay(5);
  }
  else if (digitalRead(sysexpin[6]) == 1 && debounce[6] == 1)
  {
    debounce[6] = 0;
    delay(5);
  }

  while (usbMIDI.read()) {}   // discard all incoming MIDI messages
}
